# %%
from pathlib import Path

from aind_mri_targeting.headframe_rotations import headframes_centers_of_mass

# %%
# Input files
basepath = Path("/path/to/data")
mri_path = basepath / "mri.nii.gz"
seg_path = basepath / "segmentation.seg.nrrd"

# Output directory
output_dir = "/path/to/output"

# Optional mouse ID
mouse_id = None

# Whether to ovewrite:
force = False

# Default key format is "{}_{}" for orientation and AP direction.
# Must match the segments in the segmentation file. If not, specify the key
# format.
segment_format = None

ignore_list = []  # list of segment names to ignore
# %%
headframes_centers_of_mass(
    mri_path,
    seg_path,
    output_dir,
    mouse_id=mouse_id,
    segment_format=segment_format,
    force=force,
    ignore_list=ignore_list,
)



"""
utilities
"""

import glob
import os
import shutil
from pathlib import Path


def install_notebooks(output=None, force=False):
    """
    Install notebooks by copying them to the specified output directory.

    Parameters
    ----------
    output : str or None, optional
        The directory where the notebooks should be installed. If None, the
        current working directory is used.  The default is None.
    force : bool, optional
        If True, overwrite existing files in the output directory. If False,
        skip copying if the file already exists.  The default is False.

    Returns
    -------
    None
        This function does not return any value.

    Notes
    -----
    - The function looks for notebooks in the 'notebooks' directory, which is
      three levels up from the location of this script.
    - If the output directory does not exist, it will be created.
    - Only files with a `.py` extension in the source 'notebooks' directory
      will be copied.
    - Existing files in the output directory will be skipped unless `force` is
      set to True.

    Examples
    --------
    Install notebooks to the current working directory, creating a 'notebooks'
    directory if it doesn't exist:

    >>> install_notebooks()

    Install notebooks to a specified directory:

    >>> install_notebooks(output='/path/to/destination')

    Force overwrite existing notebooks in the specified directory:

    >>> install_notebooks(output='/path/to/destination', force=True)
    """
    if output is None:
        output = os.getcwd()
    output_dir = Path(output) / "notebooks"
    notebook_dir = Path(__file__).resolve().parent.parent.parent / "notebooks"
    files_to_write = glob.glob(str(notebook_dir / "*.py"))
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    files_in_output = set(os.listdir(output_dir))
    for file in files_to_write:
        filename = Path(file).name
        if filename in files_in_output and not force:
            print(f"Skipping {filename}")
            continue
        print(f"Copying {filename}")
        shutil.copy2(str(file), str(output_dir))
    return


def check_output_path(output_path=None):
    """
    Check if the provided output path is a valid directory.

    If no output path is provided, the current working directory is used.

    Parameters
    ----------
    output_path : str, optional
        The path to the output directory. Defaults to None.

    Returns
    -------
    pathlib.Path
        The output path as a `Path` object.

    Raises
    ------
    NotADirectoryError
        If the output path is not a directory.
    """
    if output_path is None:
        output_path = os.getcwd()
    if not os.path.isdir(output_path):
        raise NotADirectoryError(
            f"Output path {output_path} is not a directory"
        )
    return Path(output_path)


def err_if_files_exist(
    files, error_message="File {} already exists. Set force=True to overwrite."
):
    """
    Check if the given files already exist and raise a FileExistsError if any
    of them do.

    Parameters
    ----------
    files : list
        A list of file paths to check for existence.
    error_message : str, optional
        The error message to raise if a file exists. Defaults to "File {}
        already exists. Set force=True to overwrite."

    Raises
    ------
    FileExistsError
        If any of the files already exist.

    Returns
    -------
    None

    Examples
    --------
    >>> err_if_files_exist(["file1.txt", "file2.txt"])
    Traceback (most recent call last):
        ...
    FileExistsError: File file1.txt already exists. Set force=True to
    overwrite.
    """
    for file in files:
        if os.path.exists(file):
            raise FileExistsError(error_message.format(file))
    return



"""
Command line script to calculate find candidate headframe transforms
"""

import argparse

from .. import headframe_rotations as hr


def parse_args():
    parser = argparse.ArgumentParser(
        description="Calculate headframe transforms from MRI and segmentation"
    )
    parser.add_argument("mri", help="path to the MRI file")
    parser.add_argument("segmentation", help="path to the segmentation file")
    parser.add_argument("lower_plane", help="path to the lower plane file")
    parser.add_argument("output", nargs="?", help="path to the output file")
    parser.add_argument("-m", "--mouse", default=None, help="mouse ID")
    parser.add_argument(
        "-s", "--segment_format", default=None, help="segment name format"
    )
    parser.add_argument(
        "-v",
        "--volume-transform",
        action="store_true",
        default=True,
        help="write transform for volume",
    )
    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        default=False,
        help="force overwrite",
    )
    parser.add_argument(
        "-i",
        "--ignore",
        nargs="+",
        default=[],
        help="list of segment names to ignore",
    )
    return parser.parse_args()


def main():
    args = parse_args()
    hr.calculate_headframe_transforms(
        args.mri,
        args.segmentation,
        args.lower_plane,
        args.output,
        mouse_name=args.mouse,
        segment_format=args.segment_format,
        force=args.force,
        volume_transforms=args.volume_transform,
        ignore_list=args.ignore,
    )
    return 0


if __name__ == "__main__":
    main()



# %%
from pathlib import Path

import numpy as np
import SimpleITK as sitk
from aind_mri_utils import reticle_calibrations as rc
from aind_mri_utils import rotations as rot
from aind_mri_utils.chemical_shift import (
    chemical_shift_transform,
    compute_chemical_shift,
)
from aind_mri_utils.file_io import simpleitk as mr_sitk
from aind_mri_utils.file_io import slicer_files as sf
from aind_mri_utils.planning import candidate_insertions

# %%
mouse = "721685"
whoami = "galen"
if whoami == "galen":
    base_dir = Path("/mnt/aind1-vast/scratch/")
    base_save_dir = Path("/home/galen.lynch/")
elif whoami == "yoni":
    base_dir = Path("Y:/")
    base_save_dir = Path(
        "C:/Users/yoni.browning/OneDrive - Allen Institute/Desktop/"
    )
else:
    raise ValueError("Who are you again?")

headframe_model_dir = base_dir / "ephys/persist/data/MRI/HeadframeModels/"
probe_model_file = (
    headframe_model_dir / "dovetailtweezer_oneShank_centered_corrected.obj"
)  # "modified_probe_holder.obj"
annotations_path = base_dir / "ephys/persist/data/MRI/processed/{}/HF".format(
    mouse
)

headframe_path = headframe_model_dir / "TenRunHeadframe.obj"
holes_path = headframe_model_dir / "OneOff_HolesOnly.obj"


implant_holes_path = str(
    annotations_path / "{}_ImplantHoles.seg.nrrd".format(mouse)
)

image_path = str(
    annotations_path / "{}_100.nii.gz".format(mouse)
)  # '_100.nii.gz'))
labels_path = str(
    annotations_path / "{}_HeadframeHoles.seg.nrrd".format(mouse)
)  # 'Segmentation.seg.nrrd')#
brain_mask_path = str(
    annotations_path / ("{}_auto_skull_strip.nrrd".format(mouse))
)
manual_annotation_path = str(
    annotations_path / (f"{mouse}_ManualAnnotations.fcsv")
)
cone_path = (
    base_dir
    / "ephys/persist/Software/PinpointBuilds/WavefrontFiles/Cone_0160-200-53.obj"  # noqa E501
)

uw_yoni_annotation_path = (
    annotations_path / f"targets-{mouse}-transformed.fcsv"
)

newscale_file_name = headframe_path / "Centered_Newscale_2pt0.obj"
#


calibration_filename = "calibration_info_np2_2024_08_13T09_57_00.xlsx"
calibration_dir = (
    base_dir / "ephys/persist/data/probe_calibrations/CSVCalibrations/"
)
calibration_file = calibration_dir / calibration_filename
measured_hole_centers = (
    annotations_path / "measured_hole_centers_conflict.xlsx"
)


# manual_hole_centers_file = annotations_path / 'hole_centers.mrk.json'

transformed_targets_save_path = annotations_path / (
    mouse + "TransformedTargets.csv"
)
test_probe_translation_save_path = str(
    base_save_dir / "test_probe_translation.h5"
)
transform_filename = str(annotations_path / (mouse + "_com_plane.h5"))

# %%
# measurement_df = pd.read_excel(measured_hole_centers)

# %%
target_structure_pair = [("4", "CCant"), ("3", "CCant")]

# %%

(
    adjusted_pairs_by_probe,
    global_offset,
    global_rotation_degrees,
    reticle_name,
) = rc.read_reticle_calibration(calibration_file)


# %%
measurements = {
    46110: {
        "3": np.array(
            [
                [8301, 7747, 8700],
            ],
            dtype=float,
        ),
    },
}
probe_scales = {46100: False, 46110: False}
# %%
transform_rs = dict()
transform_offsets = dict()
scaling_vecs = dict()
for probe in measurements:
    reticle_coords, probe_coords = adjusted_pairs_by_probe[probe]
    if probe_scales[probe]:
        (
            transform_rs[probe],
            scaling_vecs[probe],
            transform_offsets[probe],
            _,
        ) = rc.fit_rotation_params(
            reticle_pts=reticle_coords,
            probe_pts=probe_coords,
            find_scaling=True,
        )
    else:
        transform_rs[probe], transform_offsets[probe], _ = (
            rc.fit_rotation_params(
                reticle_pts=reticle_coords,
                probe_pts=probe_coords,
                find_scaling=False,
            )
        )
        scaling_vecs[probe] = None

# %%
transformed_global_points = dict()
for probe in measurements:
    transformed_global_points[probe] = dict()
    for name, coord in measurements[probe].items():
        transformed_global_points[probe][name] = rc.transform_probe_to_reticle(
            measurements[probe][name] / 1000,
            transform_rs[probe],
            transform_offsets[probe],
            scale=scaling_vecs[probe],
        )
# %%
manual_annotation = sf.read_slicer_fcsv(manual_annotation_path)
image = sitk.ReadImage(image_path)
trans = mr_sitk.load_sitk_transform(
    transform_filename, homogeneous=True, invert=True
)[0]


# %%
chem_shift = compute_chemical_shift(image)
chem_shift_trans = chemical_shift_transform(chem_shift, readout="HF")
# -

# List targeted locations
preferred_pts = {k[1]: manual_annotation[k[1]] for k in target_structure_pair}

hmg_pts = rot.prepare_data_for_homogeneous_transform(
    np.array(tuple(preferred_pts.values()))
)
chem_shift_annotation = hmg_pts @ trans.T @ chem_shift_trans.T
transformed_annotation = rot.extract_data_for_homogeneous_transform(
    chem_shift_annotation
)
target_names = tuple(preferred_pts.keys())

# %%
transformed_global_points[46110]
df = candidate_insertions(
    transformed_annotation,
    transformed_global_points[46110]["3"] * np.array([-1, -1, 1]),
    target_names,
    ["3"],
)

# %%



# %%
# Imports
# %%
import importlib
import itertools as itr

import numpy as np
from aind_mri_utils import parallax_calibration as pc
from aind_mri_utils import reticle_calibrations as rc
from matplotlib import pyplot as plt

importlib.reload(rc)
# %%
# Define the calibration file and probe
cal_file = "/mnt/aind1-vast/scratch/ephys/persist/data/probe_calibrations/CSVCalibrations/calibration_info_np2_2024_08_01T11_23_00.xlsx"  # noqa E501
probe = 46116


# %%
# Functions
def calc_pairwise_distances(pts):
    npt = pts.shape[0]
    pairwise_idxs = list(itr.product(range(npt), range(npt)))

    distances = np.empty((npt, npt))

    for i, j in pairwise_idxs:
        distances[i, j] = np.linalg.norm(pts[i, :] - pts[j, :])
    return distances


# %%
# Load the calibration file
(
    adjusted_pairs_by_probe,
    global_offset,
    global_rotation_degrees,
    reticle_name,
) = rc.read_reticle_calibration(cal_file)

reticle_pts, probe_pts = adjusted_pairs_by_probe[probe]

# %%
reticle_distances = calc_pairwise_distances(reticle_pts)
probe_distances = calc_pairwise_distances(probe_pts)
# %%
drift = reticle_distances - probe_distances

# %%
term_kwargs = dict(xtol=1e-12, gtol=1e-12, ftol=1e-15)
R, scaling, translation, res = rc.fit_rotation_params(
    reticle_pts, probe_pts, find_scaling=True, **term_kwargs
)
R_unscaled, translation_unscaled, res_unscaled = rc.fit_rotation_params(
    reticle_pts, probe_pts, find_scaling=False, **term_kwargs
)

rt = pc.RotationTransformation()
translation_parallax, R_parallax, scale_parallax, _ = rt.fit_params(
    probe_pts, reticle_pts
)
# %%
fit_reticle_pts = rc.transform_probe_to_reticle(
    probe_pts, R, translation, scaling
)
fit_reticle_pts_unscaled = rc.transform_probe_to_reticle(
    probe_pts, R_unscaled, translation_unscaled
)
fit_probe_pts = rc.transform_reticle_to_probe(
    reticle_pts, R, translation, scaling
)
fit_probe_pts_unscaled = rc.transform_reticle_to_probe(
    reticle_pts, R_unscaled, translation_unscaled
)
# %%
f, ax = plt.subplots(1, 1, figsize=(10, 10), subplot_kw=dict(projection="3d"))
# %%
errs = fit_reticle_pts - reticle_pts
errs_unscaled = fit_reticle_pts_unscaled - reticle_pts
errs_probe = fit_probe_pts - probe_pts
errs_probe_unscaled = fit_probe_pts_unscaled - probe_pts
med_err = np.median(np.linalg.norm(errs, axis=1))
med_err_unscaled = np.median(np.linalg.norm(errs_unscaled, axis=1))
med_err_probe = np.median(np.linalg.norm(errs_probe, axis=1))
med_err_probe_unscaled = np.median(np.linalg.norm(errs_probe_unscaled, axis=1))
X, Y, Z = [reticle_pts[:, i] for i in range(3)]
U, V, W = [errs[:, i] for i in range(3)]
# ax.quiver3D(X, Y, Z, U, V, W, angles="xy", scale_units="xy", scale=1)
# %%
probe_distances_scaled = calc_pairwise_distances(probe_pts * scaling)
drift_scaled = reticle_distances - probe_distances_scaled

# %%
